@@ -1,7 +1,9 @@
 package Mesos::Scheduler;
-use Mesos::Messages;
 use Moo;
-use namespace::autoclean;
+use strict;
+use warnings;
+
+with 'Mesos::Role::Scheduler';
 
 sub registered {}
 sub reregistered {}
@@ -15,15 +17,15 @@ sub executorLost {}
 sub error {}
 
 
-=head1 NAME
+=head1 Name
 
 Mesos::Scheduler - base class for Mesos schedulers
 
-=head1 SYNOPSIS
+=head1 Synopsis
 
 Mesos::Scheduler methods are callbacks which will are invoked by Mesos::SchedulerDriver.
 
-=head1 METHODS
+=head1 Methods
 
 =over 4
 
